package _mapping_consistency;

import ronmodel.Codomain;
import ronmodel.Mapping;
import ronmodel.ONNode;

/**
 * @author Alex
 * 
 * Returnvalue of mappingConsistencyCheck
 *
 */
public class MappingConsistencyResult {
	
	public static final int CONSISTENT=0;
	public static final int LEFT_TOTALITY_VIOLATED=1; // is also consistent
	
	public static final int INJECTIVITY_VIOLATED=2;	
	public static final int PRE_COND_VIOLATED=3;
	public static final int POST_COND_VIOLATED=4;
	public static final int TYPES_VIOLATED=5;
	public static final int INTERNAL_STRUCTURE_FAULT=6; //if mapping is not initialized fully
			
	private final Codomain codomain; //codomain of mapping
	
	private int errorCode=CONSISTENT; //errorcode
	private String messageWithNames="Mapping is consistent!"; //error-message with names
	private String messageWithMappingNrs="Mapping is consistent!"; //error-message with mapping-numbers
	private Mapping mapping=null; //inconsistent mapping
	private ONNode errorNode=null; //the nodes that produces the error
	private int mappingNr=-1; //the mapping nr or -1 if there is none
	
	/**
	 * @param codomain codomain for mapping
	 */
	public MappingConsistencyResult(Codomain codomain){
		this.codomain=codomain;
	}
	
	/**
	 * @param errorCode new errorcode (see constants)
	 */
	public void setResult(int errorCode){
		this.errorCode=errorCode;
	}
	
	/**
	 * @param errorNode the node that produces the error
	 */
	public void setErrorNode(ONNode errorNode){
		this.errorNode=errorNode;
	}
	
	/**
	 * @return the node that produces the error, or null
	 */
	public ONNode getErrorNode(){
		return errorNode;
	}
	
	/**
	 * @param m mapping which is inconsistent
	 */
	public void setMapping(Mapping m){
		mapping=m;
	}
	
	/**
	 * @return inconsistent mapping or null
	 */
	public Mapping getMapping(){
		return mapping;
	}
	
	/**
	 * @return errorCode (see constants)
	 */
	public int getResult(){
		return errorCode;
	}
	
	/**
	 * @param msg message for user
	 */
	public void setMessageWithNames(String msg){
		messageWithNames=msg;
	}
	
	/**
	 * @param msg message for user
	 */
	public void setMessageWithMappingNrs(String msg){
		messageWithMappingNrs=msg;
	}
	
	/**
	 * @return message for user
	 */
	public String getMessageWithNames(){
		return messageWithNames;
	}
	
	/**
	 * @return message for user
	 */
	public String getMessageWithMappingNrs(){
		return messageWithMappingNrs;
	}
	
	/**
	 * @return codomain of mapping
	 */
	public Codomain getCodomain(){
		return codomain;
	}
	
	/**
	 * Returns if the mapping is consistent.
	 * A Mapping is consistent, if and only if
	 * 	- it is injective
	 * 	- pre-condition is satisfied
	 *  - post condition is satisfied
	 * Note: (mapping must not be left total)
	 * @return true if the mapping is consistent ), otherwise false
	 */
	public boolean isConsistent(){
		return (errorCode==CONSISTENT || errorCode==LEFT_TOTALITY_VIOLATED);
	}
	
	/**
	 * If this method returns true, the message should not be visible for the
	 * user!
	 * @return true if there is an internal structure fault (the mapping
	 * is not initialized fully)
	 */
	public boolean internalStructureFault(){
		return (errorCode==INTERNAL_STRUCTURE_FAULT);
	}
	
	/* Informations about the result.
	 *  
	 * Use only for debugging!
	 */
	@Override
	public String toString(){
		if (errorCode==CONSISTENT) return "Mapping "+mapping+" is consistent.";
		else return "Mapping "+mapping+" is inconsistent.";
	}

	/**
	 * @return the mappingNr
	 */
	public int getMappingNr() {
		return mappingNr;
	}

	/**
	 * @param mappingNr the mappingNr to set
	 */
	public void setMappingNr(int mappingNr) {
		this.mappingNr = mappingNr;
	}
	
	/**
	 * @return true if mapping nr is valid (>0)
	 */
	public boolean isValidMappingNr(){
		return (mappingNr>0);
	}

}
