/**
 * 
 */
package animation;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.UpdateManager;

public class MultipleAnimation {

	public static class AnimPair {

		final AnimatorForMultiple animator;
		final IFigure figure;

		AnimPair(final AnimatorForMultiple animator, final IFigure figure) {
			this.animator = animator;
			this.figure = figure;
		}

		@Override
		public boolean equals(final Object obj) {
			final MultipleAnimation.AnimPair pair = (MultipleAnimation.AnimPair) obj;
			return pair.animator == animator && pair.figure == figure;
		}

		@Override
		public int hashCode() {
			return animator.hashCode() ^ figure.hashCode();
		}
	}

	private static final int DEFAULT_DELAY = 250;
	private static Set<MultipleAnimation.AnimPair> figureAnimators;
	private static Map<MultipleAnimation.AnimPair, Object> finalStates;

	private static Map<MultipleAnimation.AnimPair, Object> initialStates;
	private static final int PLAYBACK = 3;
	private static float progress;
	private static final int RECORD_FINAL = 2;

	private static final int RECORD_INITIAL = 1;
	private static long startTime;
	private static int state;
	private static Set<MultipleAnimation.AnimPair> toCapture;

	// private static UpdateManager updateManager;
	private static Set<UpdateManager> updateManagers = new HashSet<UpdateManager>();
	
	private static void capture() {
		final Iterator<MultipleAnimation.AnimPair> keys = figureAnimators.iterator();
		while (keys.hasNext()) {
			final MultipleAnimation.AnimPair pair = keys.next();
			if (toCapture.contains(pair)) {
				pair.animator.capture(pair.figure);
			} else {
				keys.remove();
			}
		}
	}

	static void cleanup() {
		state = 0;
		step();
		// Allow layout to occur normally
		// updateManager.performUpdate();

		initialStates = null;
		finalStates = null;
		figureAnimators = null;
		//updateManager = null;
		updateManagers.clear();
		
		toCapture = null;
		state = 0;
	}

	private static void doRun(final int duration) {
		state = RECORD_FINAL;
		findUpdateManager();

		// No validation is done, instead Animation.hookNeedsCapture(parent,
		// Animator.getDefault()) is called in animate()
		// updateManager.performValidation();
		capture();
		state = PLAYBACK;
		progress = 0.1f;
		startTime = System.currentTimeMillis();

		while (progress != 0) {
			step();
			
			//updateManager.performUpdate();
			for (UpdateManager updateManager : updateManagers) {
				updateManager.performUpdate();
			}
			
			
			if (progress == 1.0) {
				progress = 0;
			} else {
				final int delta = (int) (System.currentTimeMillis() - startTime);
				if (delta >= duration) {
					progress = 1f;
				} else {
					progress = 0.1f + 0.9f * delta / duration;
				}
			}
		}
	}

	private static void findUpdateManager() {
//		final MultipleAnimation.AnimPair pair = figureAnimators.iterator().next();
//		updateManager = pair.figure.getUpdateManager();
		
		for (MultipleAnimation.AnimPair pair : figureAnimators) {
			updateManagers.add(pair.figure.getUpdateManager());
		}
		
	}

	/**
	 * Returns the final animation state for the given figure.
	 * 
	 * @param animator
	 *            the animator for the figure
	 * @param figure
	 *            the figure being animated
	 * @return the final state
	 * @since 3.2
	 */
	public static Object getFinalState(final AnimatorForMultiple animator,
			final IFigure figure) {
		return finalStates.get(new AnimPair(animator, figure));
	}

	/**
	 * Returns the initial animation state for the given animator and
	 * figure. If no state was recorded, <code>null</code> is returned.
	 * 
	 * @param animator
	 *            the animator for the figure
	 * @param figure
	 *            the figure being animated
	 * @return the initial state
	 * @since 3.2
	 */
	public static Object getInitialState(final AnimatorForMultiple animator,
			final IFigure figure) {
		return initialStates.get(new AnimPair(animator, figure));
	}

	/**
	 * Returns the animation progress, where 0.0 < progress &#8804; 1.0.
	 * 
	 * @return the progress of the animation
	 * @since 3.2
	 */
	public static float getProgress() {
		return progress;
	}

	@SuppressWarnings("unchecked")
	static void hookAnimator(final IFigure figure, final AnimatorForMultiple animator) {
		final MultipleAnimation.AnimPair pair = new AnimPair(animator, figure);
		if (figureAnimators.add(pair)) {
			animator.init(figure);
		}
	}

	@SuppressWarnings("unchecked")
	static void hookNeedsCapture(final IFigure figure,
			final AnimatorForMultiple animator) {
		final MultipleAnimation.AnimPair pair = new AnimPair(animator, figure);
		if (figureAnimators.contains(pair)) {
			toCapture.add(pair);
		}
	}

	static boolean hookPlayback(final IFigure figure,
			final AnimatorForMultiple animator) {
		if (toCapture.contains(new AnimPair(animator, figure))) {
			return animator.playback(figure);
		}
		return false;
	}

	/**
	 * Returns <code>true</code> if animation is in progress.
	 * 
	 * @return <code>true</code> when animating
	 * @since 3.2
	 */
	public static boolean isAnimating() {
		return state == PLAYBACK;
	}

	static boolean isFinalRecording() {
		return state == RECORD_FINAL;
	}

	static boolean isInitialRecording() {
		return state == RECORD_INITIAL;
	}

	/**
	 * Marks the beginning of the animation process. If the beginning has
	 * already been marked, this has no effect.
	 * 
	 * @return returns <code>true</code> if beginning was not previously
	 *         marked
	 * @since 3.2
	 */
	public static boolean markBegin() {
		if (state == 0) {
			state = RECORD_INITIAL;
			initialStates = new HashMap<MultipleAnimation.AnimPair, Object>();
			finalStates = new HashMap<MultipleAnimation.AnimPair, Object>();
			figureAnimators = new HashSet<MultipleAnimation.AnimPair>();
			toCapture = new HashSet<MultipleAnimation.AnimPair>();
			return true;
		}
		return false;
	}

	@SuppressWarnings("unchecked")
	static void putFinalState(final AnimatorForMultiple animator, final IFigure key,
			final Object state) {
		finalStates.put(new AnimPair(animator, key), state);
	}

	@SuppressWarnings("unchecked")
	static void putInitialState(final AnimatorForMultiple animator, final IFigure key,
			final Object state) {
		initialStates.put(new AnimPair(animator, key), state);
	}

	/**
	 * Runs animation using the recommended duration: 250 milliseconds.
	 * 
	 * @see #run(int)
	 * @since 3.2
	 */
	public static void run() {
		run(DEFAULT_DELAY);
	}

	/**
	 * Captures the final states for the animation and then plays the
	 * animation.
	 * 
	 * @param duration
	 *            the length of animation in milliseconds
	 * @since 3.2
	 */
	public static void run(final int duration) {
		if (state == 0) {
			return;
		}
		try {
			if (!figureAnimators.isEmpty()) {
				doRun(duration);
			}
		} finally {
			cleanup();
		}
	}

	private static void step() {
		final Iterator<MultipleAnimation.AnimPair> iter = initialStates.keySet().iterator();
		while (iter.hasNext()) {
			iter.next().figure.revalidate();
		}
	}

}