package roneditor.editor;

import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;

/**
 * This ContextMenuProvider gets an {@link ActionRegistry} in its constructor
 * which is used to resolve actions via some key, usually their IDs. Some
 * methods are provided to append (dynamically) an action several standard GEF
 * action groups.
 * 
 * @author Tony Modica
 */
public abstract class ContextMenuProviderWithActionRegistry extends
		ContextMenuProvider {

	/**
	 * The {@link ActionRegistry} used to resolve the actions.
	 */
	private ActionRegistry actionRegistry;

	/**
	 * The constructor for this class.
	 * 
	 * @param viewer
	 *            The viewer in which the context menu is used.
	 * @param actionRegistry
	 *            An action registry hosting the actions to show in the context
	 *            menu.
	 */
	public ContextMenuProviderWithActionRegistry(final EditPartViewer viewer,
			final ActionRegistry actionRegistry) {
		super(viewer);
		this.actionRegistry = actionRegistry;
	}

	/**
	 * Appends the specified action to the specified group.
	 * 
	 * @param menu
	 *            The MenuManager to be used.
	 * @param actionId
	 *            The id of the action to be put in the context menu.
	 * @param group
	 *            The group to put in the action.
	 */
	protected void appendActionToGroup(final IMenuManager menu,
			final String actionId, final String group) {
		final IAction action = actionRegistry.getAction(actionId);
		if (null != action) {
			menu.appendToGroup(group, action);
		}
	}

	/**
	 * Appends the specified action to the menu edit group if the action is
	 * enabled.
	 * 
	 * @param menu
	 *            The MenuManager to be used.
	 * @param actionId
	 *            The id of the action to be put in the context menu.
	 * @param group
	 *            The group to put in the action.
	 */
	protected void dynamicAppendActionToGroup(final IMenuManager menu,
			final String actionId, final String group) {
		final IAction action = actionRegistry.getAction(actionId);
		if (null != action && action.isEnabled()) {
			menu.appendToGroup(group, action);
		}
	}
}